diff --git a/src/main.cpp b/src/main.cpp
index f075504..be62e61 100644
--- a/src/main.cpp
+++ b/src/main.cpp
@@ -46,6 +46,8 @@ private:
     vk::raii::Instance m_instance{ nullptr };
     vk::raii::DebugUtilsMessengerEXT m_debugMessenger{ nullptr };
     vk::raii::PhysicalDevice m_physicalDevice{ nullptr };
+    vk::raii::Device m_device{ nullptr };
+    vk::raii::Queue m_graphicsQueue{ nullptr };
     /////////////////////////////////////////////////////////////////
 
     /////////////////////////////////////////////////////////////////
@@ -63,6 +65,7 @@ private:
         createInstance();
         setupDebugMessenger();
         pickPhysicalDevice();
+        createLogicalDevice();
     }
 
     void mainLoop() {
@@ -225,6 +228,32 @@ private:
         return indices;
     }
     /////////////////////////////////////////////////////////////////
+
+    /////////////////////////////////////////////////////////////////
+    /// logical device
+    void createLogicalDevice() {
+        QueueFamilyIndices indices = findQueueFamilies( m_physicalDevice );
+
+        vk::DeviceQueueCreateInfo queueCreateInfo;
+        queueCreateInfo.queueFamilyIndex = indices.graphicsFamily.value();
+
+        float queuePriority = 1.0f;
+        queueCreateInfo.setQueuePriorities( queuePriority );
+
+        vk::PhysicalDeviceFeatures deviceFeatures;
+
+        vk::DeviceCreateInfo createInfo;
+        createInfo.setQueueCreateInfos( queueCreateInfo );
+        createInfo.pEnabledFeatures = &deviceFeatures;
+
+        if (enableValidationLayers) {
+            createInfo.setPEnabledLayerNames( validationLayers );
+        }
+
+        m_device = m_physicalDevice.createDevice( createInfo );
+        m_graphicsQueue = m_device.getQueue( indices.graphicsFamily.value(), 0 );
+    }
+    /////////////////////////////////////////////////////////////////
 };
 
 int main() {
